#include "stc.h"
#include "usb.h"
#include "usb_desc.h"
#include "usb_req_class.h"
#include "keycode.h"

BYTE bHidIdle;
BYTE bKeyCode = 0;
BYTE bKeyDebounce = 0;
BOOL fKeyOK;

sbit LED_NUM        = P6^7;
sbit LED_CAPS       = P6^6;
sbit LED_SCROLL     = P6^5;
sbit LED_TEST1      = P6^0;

void usb_req_class()
{
    switch (Setup.bRequest)
    {
    case GET_REPORT:
        usb_get_report();
        break;
    case SET_REPORT:
        usb_set_report();
        break;
    case GET_IDLE:
        usb_get_idle();
        break;
    case SET_IDLE:
        usb_set_idle();
        break;
    case GET_PROTOCOL:
        usb_get_protocol();
        break;
    case SET_PROTOCOL:
        usb_set_protocol();
        break;
    default:
        usb_setup_stall();
        return;
    }
}

void usb_get_report()
{
    if ((DeviceState != DEVSTATE_CONFIGURED) ||
        (Setup.bmRequestType != (IN_DIRECT | CLASS_REQUEST | INTERFACE_RECIPIENT)))
    {
        usb_setup_stall();
        return;
    }

    Ep0State.pData = UsbBuffer;
    Ep0State.wSize = Setup.wLength;

    usb_setup_in();
}

void usb_set_report()
{
    if ((DeviceState != DEVSTATE_CONFIGURED) ||
        (Setup.bmRequestType != (OUT_DIRECT | CLASS_REQUEST | INTERFACE_RECIPIENT)))
    {
        usb_setup_stall();
        return;
    }

    Ep0State.pData = UsbBuffer;
    Ep0State.wSize = Setup.wLength;

    usb_setup_out();
}

void usb_get_idle()
{
    if ((DeviceState != DEVSTATE_CONFIGURED) ||
        (Setup.bmRequestType != (IN_DIRECT | CLASS_REQUEST | INTERFACE_RECIPIENT)))
    {
        usb_setup_stall();
        return;
    }

    Ep0State.pData = &bHidIdle;
    Ep0State.wSize = 1;

    usb_setup_in();
}

void usb_set_idle()
{
    if ((DeviceState != DEVSTATE_CONFIGURED) ||
        (Setup.bmRequestType != (OUT_DIRECT | CLASS_REQUEST | INTERFACE_RECIPIENT)))
    {
        usb_setup_stall();
        return;
    }

    bHidIdle = Setup.wValueH;

    usb_setup_status();
}

void usb_get_protocol()
{
    Ep0State.pData = PACKET0;
    Ep0State.wSize = 1;

    usb_setup_in();
}

void usb_set_protocol()
{
    usb_setup_status();
}

void usb_class_out()
{
    BYTE led;
    
    if (usb_bulk_intr_out(UsbBuffer, 1) == 1)
    {
        P4M0 &= ~0x01;
        P4M1 &= ~0x01;
        P6M0 &= ~0xff;
        P6M1 &= ~0xff;
        P40 = 0;
        
        led = UsbBuffer[0];
        LED_NUM = !(led & 0x01);
        LED_CAPS = !(led & 0x02);
        LED_SCROLL = !(led & 0x04);
    }
}

static void delay()
{
    int i;
    
    for (i = 0; i < 3000; i++)
    {
        _nop_();
        _nop_();
        _nop_();
        _nop_();
    }
}

void usb_class_in()
{
    static BYTE id = 0;
    BYTE key[9];
    BYTE i;
    
    if (DeviceState != DEVSTATE_CONFIGURED)
        return;

    if (!UsbInBusy && fKeyOK)
    {
        fKeyOK = 0;

        key[0] = 1;
        key[1] = 0;
        key[2] = 0;
        key[3] = 0;
        key[4] = 0;
        key[5] = 0;
        key[6] = 0;
        key[7] = 0;
        key[8] = 0;

        switch (bKeyCode)
        {
        case 0xfe:
            id = 1;
            key[3] = ID_KB_1;
            break;
        case 0xfd:
            id = 1;
            key[3] = ID_KB_2;
            break;
        case 0xfb:
            id = 1;
            key[3] = ID_KB_A;
            break;
        case 0xf7:
            id = 1;
            key[3] = ID_KB_B;
            break;
        case 0xef:
            id = 2;
            key[1] = ID_MK_VolumeInc;
            key[2] = ID_MK_VolumeInc >> 8;
            break;
        case 0xdf:
            id = 2;
            key[1] = ID_MK_VolumeDec;
            key[2] = ID_MK_VolumeDec >> 8;
            break;
        case 0xbf:
            id = 3;
            key[1] = ID_KB_Sleep;
            break;
        case 0x7f:
            id = 3;
            key[1] = ID_KB_Wakeup;
            break;
        }
        key[0] = id;
        
        IE2 &= ~0x80;
        if (usb_read_reg(POWER) & 0x02)
        {
            LED_TEST1 = 0;
            usb_write_reg(POWER, 5);
            delay();
            usb_write_reg(POWER, 1);
        }
        
        UsbInBusy = 1;
        usb_write_reg(INDEX, 1);
        for (i=0; i<9; i++)
        {
            usb_write_reg(FIFO1, key[i]);
        }
        usb_write_reg(INCSR1, INIPRDY);
        IE2 |= 0x80;
    }
}

void scan_key()
{
    BYTE key;
    
    P0M0 = 0x00;
    P0M1 = 0x00;
    
    key = 0;
    P0 = 0xff;
    P06 = 0;
    _nop_();
    _nop_();
    key |= P0 & 0x0f;
    P06 = 1;
    P07 = 0;
    _nop_();
    _nop_();
    key |= (P0 & 0x0f) << 4;
    P07 = 1;
    
    if (key != bKeyCode)
    {
        bKeyCode = key;
        bKeyDebounce = 20;
    }
    else
    {
        if (bKeyDebounce)
        {
            bKeyDebounce--;
            if (bKeyDebounce == 0)
            {
                fKeyOK = 1;
            }
        }
    }
    
}

